(** OCaml bindings for libvirt.
    (C) Copyright 2007 Richard W.M. Jones, Red Hat Inc.
    http://libvirt.org/
    $Id: libvirt.mli,v 1.9 2007/06/23 10:17:19 rjones Exp $
*)

type uuid = string
(** This is a "raw" UUID, ie. a packed string of bytes. *)

type xml = string
(** Type of XML (an uninterpreted string of bytes).  Use PXP, expat,
    xml-light, etc. if you want to do anything useful with the XML.
*)

val get_version : ?driver:string -> unit -> int * int
  (** [get_version ()] returns the library version in the first part
      of the tuple, and zero in the second part.

      [get_version ~driver ()] returns the library version in the first
      part of the tuple, and the version of the driver called [type]
      in the second part.

      The version numbers are encoded as
      1,000,000 * major + 1,000 * minor + release.
  *)

val uuid_length : int
  (** Length of packed UUIDs. *)

val uuid_string_length : int
  (** Length of UUID strings. *)

module Connect :
sig
  type t
    (** Connection. *)

  type node_info = {
    model : string;			(** CPU model *)
    memory : int64;			(** memory size in kilobytes *)
    cpus : int;				(** number of active CPUs *)
    mhz : int;				(** expected CPU frequency *)
    nodes : int;			(** number of NUMA nodes (1 = UMA) *)
    sockets : int;			(** number of CPU sockets per node *)
    cores : int;			(** number of cores per socket *)
    threads : int;			(** number of threads per core *)
  }

  val connect : ?name:string -> ?readonly:bool -> unit -> t
    (** [connect ~name ()] connects to the hypervisor with URI [name].

	[connect ()] connects to the default hypervisor.

	Supplying [~readonly:true] connects in read-only mode.
    *)

  val close : t -> unit
    (** [close conn] closes and frees the connection object in memory.

	The connection is automatically closed if it is garbage
	collected.  This function just forces it to be closed
	and freed right away.
    *)

  val get_type : t -> string
  val get_version : t -> int
  val get_max_vcpus : t -> ?type_:string -> unit -> int
  val list_domains : t -> int -> int array
  val num_of_domains : t -> int
  val get_capabilities : t -> string
  val num_of_defined_domains : t -> int
  val list_defined_domains : t -> int -> string array
  val num_of_networks : t -> int
  val list_networks : t -> int -> string array
  val num_of_defined_networks : t -> int
  val list_defined_networks : t -> int -> string array

    (* The name of this function is inconsistent, but the inconsistency
     * is really in libvirt itself.
     *)
  val get_node_info : t -> node_info

  val maxcpus_of_node_info : node_info -> int
    (** Calculate the total number of CPUs supported (but not necessarily
	active) in the host.
    *)

  val cpumaplen : int -> int
    (** Calculate the length (in bytes) required to store the complete
	CPU map between a single virtual and all physical CPUs of a domain.
    *)

  val use_cpu : string -> int -> unit
    (** [use_cpu cpumap cpu] marks [cpu] as usable in [cpumap]. *)
  val unuse_cpu : string -> int -> unit
    (** [unuse_cpu cpumap cpu] marks [cpu] as not usable in [cpumap]. *)
  val cpu_usable : string -> int -> int -> int -> bool
    (** [cpu_usable cpumaps maplen vcpu cpu] checks returns true iff the
	[cpu] is usable by [vcpu]. *)
end
  (** Module dealing with connections.  [Connect.t] is the
      connection object.
  *)

module Domain :
sig
  type t
    (** Domain. *)

  type state =
    | InfoNoState | InfoRunning | InfoBlocked | InfoPaused
    | InfoShutdown | InfoShutoff | InfoCrashed

  type info = {
    state : state;			(** running state *)
    max_mem : int64;			(** maximum memory in kilobytes *)
    memory : int64;			(** memory used in kilobytes *)
    nr_virt_cpu : int;			(** number of virtual CPUs *)
    cpu_time : int64;			(** CPU time used in nanoseconds *)
  }

  type vcpu_state = VcpuOffline | VcpuRunning | VcpuBlocked

  type vcpu_info = {
    number : int;			(** virtual CPU number *)
    vcpu_state : vcpu_state;		(** state *)
    vcpu_time : int64;			(** CPU time used in nanoseconds *)
    cpu : int;				(** real CPU number, -1 if offline *)
  }

  type sched_param = string * sched_param_value
  and sched_param_value =
    | SchedFieldInt32 of int32 | SchedFieldUInt32 of int32
    | SchedFieldInt64 of int64 | SchedFieldUInt64 of int64
    | SchedFieldFloat of float | SchedFieldBool of bool

  val create_linux : Connect.t -> xml -> t
  val lookup_by_id : Connect.t -> int -> t
  val lookup_by_uuid : Connect.t -> uuid -> t
  val lookup_by_uuid_string : Connect.t -> string -> t
  val lookup_by_name : Connect.t -> string -> t
  val destroy : t -> unit
  val free : t -> unit
    (** [free domain] frees the domain object in memory.

	The domain object is automatically freed if it is garbage
	collected.  This function just forces it to be freed right
	away.
    *)

  val suspend : t -> unit
  val resume : t -> unit
  val save : t -> string -> unit
  val restore : Connect.t -> string -> unit
  val core_dump : t -> string -> unit
  val shutdown : t -> unit
  val reboot : t -> unit
  val get_name : t -> string
  val get_uuid : t -> uuid
  val get_uuid_string : t -> string
  val get_id : t -> int
    (** [getid dom] returns the ID of the domain.

	Do not call this on a defined but not running domain.  Those
	domains don't have IDs, and you'll get an error here.
    *)

  val get_os_type : t -> string
  val get_max_memory : t -> int64
  val set_max_memory : t -> int64 -> unit
  val set_memory : t -> int64 -> unit
  val get_info : t -> info
  val get_xml_desc : t -> xml
  val get_scheduler_type : t -> string * int
  val get_scheduler_parameters : t -> int -> sched_param array
  val set_scheduler_parameters : t -> sched_param array -> unit
  val define_xml : Connect.t -> xml -> t
  val undefine : t -> unit
  val create : t -> unit
  val get_autostart : t -> bool
  val set_autostart : t -> bool -> unit
  val set_vcpus : t -> int -> unit
  val pin_vcpu : t -> int -> string -> unit
  val get_vcpus : t -> int -> int -> int * vcpu_info array * string
  val get_max_vcpus : t -> int
  val attach_device : t -> xml -> unit
  val detach_device : t -> xml -> unit
end
  (** Module dealing with domains.  [Domain.t] is the
      domain object.
  *)

module Network : 
sig
  type t
    (** Network *)

  val lookup_by_name : Connect.t -> string -> t
  val lookup_by_uuid : Connect.t -> uuid -> t
  val lookup_by_uuid_string : Connect.t -> string -> t
  val create_xml : Connect.t -> xml -> t
  val define_xml : Connect.t -> xml -> t
  val undefine : t -> unit
  val create : t -> unit
  val destroy : t -> unit
  val free : t -> unit
    (** [free network] frees the network object in memory.

	The network object is automatically freed if it is garbage
	collected.  This function just forces it to be freed right
	away.
    *)

  val get_name : t -> string
  val get_uuid : t -> uuid
  val get_uuid_string : t -> string
  val get_xml_desc : t -> xml
  val get_bridge_name : t -> string
  val get_autostart : t -> bool
  val set_autostart : t -> bool -> unit
end
  (** Module dealing with networks.  [Network.t] is the
      network object.
  *)

module Virterror :
sig
  type t
    (** An error object. *)

    (* XXX more to follow here *)

end
  (** Module dealing with errors. *)

exception Virterror of string
(** This exception can be raised by any library function that detects
    an error.  The string is a printable error message.

    NB. We will change this to Virterror of string * Virterror.t
    at a later date.
*)

